@@ -1,8 +1,19 @@
 """Main package for humanize."""
-import pkg_resources
+
+import importlib.metadata
+
 from humanize.filesize import naturalsize
-from humanize.i18n import activate, deactivate
-from humanize.number import apnumber, fractional, intcomma, intword, ordinal, scientific
+from humanize.i18n import activate, deactivate, decimal_separator, thousands_separator
+from humanize.number import (
+    apnumber,
+    clamp,
+    fractional,
+    intcomma,
+    intword,
+    metric,
+    ordinal,
+    scientific,
+)
 from humanize.time import (
     naturaldate,
     naturalday,
@@ -11,17 +22,20 @@ from humanize.time import (
     precisedelta,
 )
 
-__version__ = VERSION = pkg_resources.get_distribution(__name__).version
+__version__ = '4.7.0'
 
 
 __all__ = [
     "__version__",
     "activate",
     "apnumber",
+    "clamp",
     "deactivate",
+    "decimal_separator",
     "fractional",
     "intcomma",
     "intword",
+    "metric",
     "naturaldate",
     "naturalday",
     "naturaldelta",
@@ -30,5 +44,5 @@ __all__ = [
     "ordinal",
     "precisedelta",
     "scientific",
-    "VERSION",
+    "thousands_separator",
 ]
