@@ -1,15 +1,35 @@
 """Activate, get and deactivate translations."""
+from __future__ import annotations
+
 import gettext as gettext_module
 import os.path
 from threading import local
 
-__all__ = ["activate", "deactivate", "gettext", "ngettext"]
+__all__ = ["activate", "deactivate", "decimal_separator", "thousands_separator"]
 
-_TRANSLATIONS = {None: gettext_module.NullTranslations()}
+_TRANSLATIONS: dict[str | None, gettext_module.NullTranslations] = {
+    None: gettext_module.NullTranslations()
+}
 _CURRENT = local()
 
 
-def _get_default_locale_path():
+# Mapping of locale to thousands separator
+_THOUSANDS_SEPARATOR = {
+    "de_DE": ".",
+    "fr_FR": " ",
+    "it_IT": ".",
+    "pt_BR": ".",
+}
+
+# Mapping of locale to decimal separator
+_DECIMAL_SEPARATOR = {
+    "de_DE": ",",
+    "it_IT": ",",
+    "pt_BR": ",",
+}
+
+
+def _get_default_locale_path() -> str | None:
     try:
         if __file__ is None:
             return None
@@ -18,14 +38,14 @@ def _get_default_locale_path():
         return None
 
 
-def get_translation():
+def get_translation() -> gettext_module.NullTranslations:
     try:
         return _TRANSLATIONS[_CURRENT.locale]
     except (AttributeError, KeyError):
         return _TRANSLATIONS[None]
 
 
-def activate(locale, path=None):
+def activate(locale: str, path: str | None = None) -> gettext_module.NullTranslations:
     """Activate internationalisation.
 
     Set `locale` as current locale. Search for locale in directory `path`.
@@ -44,10 +64,11 @@ def activate(locale, path=None):
         path = _get_default_locale_path()
 
     if path is None:
-        raise Exception(
+        msg = (
             "Humanize cannot determinate the default location of the 'locale' folder. "
             "You need to pass the path explicitly."
         )
+        raise Exception(msg)
     if locale not in _TRANSLATIONS:
         translation = gettext_module.translation("humanize", path, [locale])
         _TRANSLATIONS[locale] = translation
@@ -55,12 +76,12 @@ def activate(locale, path=None):
     return _TRANSLATIONS[locale]
 
 
-def deactivate():
+def deactivate() -> None:
     """Deactivate internationalisation."""
     _CURRENT.locale = None
 
 
-def gettext(message):
+def _gettext(message: str) -> str:
     """Get translation.
 
     Args:
@@ -72,7 +93,7 @@ def gettext(message):
     return get_translation().gettext(message)
 
 
-def pgettext(msgctxt, message):
+def _pgettext(msgctxt: str, message: str) -> str:
     """Fetches a particular translation.
 
     It works with `msgctxt` .po modifiers and allows duplicate keys with different
@@ -85,25 +106,16 @@ def pgettext(msgctxt, message):
     Returns:
         str: Translated text.
     """
-    # This GNU gettext function was added in Python 3.8, so for older versions we
-    # reimplement it. It works by joining `msgctx` and `message` by '4' byte.
-    try:
-        # Python 3.8+
-        return get_translation().pgettext(msgctxt, message)
-    except AttributeError:
-        # Python 3.7 and older
-        key = msgctxt + "\x04" + message
-        translation = get_translation().gettext(key)
-        return message if translation == key else translation
+    return get_translation().pgettext(msgctxt, message)
 
 
-def ngettext(message, plural, num):
-    """Plural version of gettext.
+def _ngettext(message: str, plural: str, num: int) -> str:
+    """Plural version of _gettext.
 
     Args:
         message (str): Singular text to translate.
         plural (str): Plural text to translate.
-        num (str): The number (e.g. item count) to determine translation for the
+        num (int): The number (e.g. item count) to determine translation for the
             respective grammatical number.
 
     Returns:
@@ -112,14 +124,14 @@ def ngettext(message, plural, num):
     return get_translation().ngettext(message, plural, num)
 
 
-def gettext_noop(message):
+def _gettext_noop(message: str) -> str:
     """Mark a string as a translation string without translating it.
 
     Example usage:
     ```python
-    CONSTANTS = [gettext_noop('first'), gettext_noop('second')]
+    CONSTANTS = [_gettext_noop('first'), _gettext_noop('second')]
     def num_name(n):
-        return gettext(CONSTANTS[n])
+        return _gettext(CONSTANTS[n])
     ```
 
     Args:
@@ -129,3 +141,49 @@ def gettext_noop(message):
         str: Original text, unchanged.
     """
     return message
+
+
+def _ngettext_noop(singular: str, plural: str) -> tuple[str, str]:
+    """Mark two strings as pluralized translations without translating them.
+
+    Example usage:
+    ```python
+    CONSTANTS = [ngettext_noop('first', 'firsts'), ngettext_noop('second', 'seconds')]
+    def num_name(n):
+        return _ngettext(*CONSTANTS[n])
+    ```
+
+    Args:
+        singular (str): Singular text to translate in the future.
+        plural (str): Plural text to translate in the future.
+
+    Returns:
+        tuple: Original text, unchanged.
+    """
+    return singular, plural
+
+
+def thousands_separator() -> str:
+    """Return the thousands separator for a locale, default to comma.
+
+    Returns:
+         str: Thousands separator.
+    """
+    try:
+        sep = _THOUSANDS_SEPARATOR[_CURRENT.locale]
+    except (AttributeError, KeyError):
+        sep = ","
+    return sep
+
+
+def decimal_separator() -> str:
+    """Return the decimal separator for a locale, default to dot.
+
+    Returns:
+         str: Decimal separator.
+    """
+    try:
+        sep = _DECIMAL_SEPARATOR[_CURRENT.locale]
+    except (AttributeError, KeyError):
+        sep = "."
+    return sep
