@@ -4,14 +4,18 @@
 
 These are largely borrowed from Django's `contrib.humanize`.
 """
+from __future__ import annotations
 
+import collections.abc
 import datetime as dt
 import math
+import typing
 from enum import Enum
 from functools import total_ordering
 
-from .i18n import gettext as _
-from .i18n import ngettext
+from .i18n import _gettext as _
+from .i18n import _ngettext
+from .number import intcomma
 
 __all__ = [
     "naturaldelta",
@@ -33,17 +37,17 @@ class Unit(Enum):
     MONTHS = 6
     YEARS = 7
 
-    def __lt__(self, other):
+    def __lt__(self, other: typing.Any) -> typing.Any:
         if self.__class__ is other.__class__:
             return self.value < other.value
         return NotImplemented
 
 
-def _now():
+def _now() -> dt.datetime:
     return dt.datetime.now()
 
 
-def abs_timedelta(delta):
+def _abs_timedelta(delta: dt.timedelta) -> dt.timedelta:
     """Return an "absolute" value for a timedelta, always representing a time distance.
 
     Args:
@@ -58,7 +62,9 @@ def abs_timedelta(delta):
     return delta
 
 
-def date_and_delta(value, *, now=None):
+def _date_and_delta(
+    value: typing.Any, *, now: dt.datetime | None = None
+) -> tuple[typing.Any, typing.Any]:
     """Turn a value into a date and a timedelta which represents how long ago it was.
 
     If that's not possible, return `(None, value)`.
@@ -78,24 +84,31 @@ def date_and_delta(value, *, now=None):
             date = now - delta
         except (ValueError, TypeError):
             return None, value
-    return date, abs_timedelta(delta)
+    return date, _abs_timedelta(delta)
 
 
-def naturaldelta(value, months=True, minimum_unit="seconds", when=None):
+def naturaldelta(
+    value: dt.timedelta | float,
+    months: bool = True,
+    minimum_unit: str = "seconds",
+) -> str:
     """Return a natural representation of a timedelta or number of seconds.
 
     This is similar to `naturaltime`, but does not add tense to the result.
 
     Args:
-        value (datetime.timedelta): A timedelta or a number of seconds.
+        value (datetime.timedelta, int or float): A timedelta or a number of seconds.
         months (bool): If `True`, then a number of months (based on 30.5 days) will be
             used for fuzziness between years.
         minimum_unit (str): The lowest unit that can be used.
-        when (datetime.timedelta): Point in time relative to which _value_ is
-            interpreted.  Defaults to the current time in the local timezone.
 
     Returns:
-        str: A natural representation of the amount of time elapsed.
+        str (str or `value`): A natural representation of the amount of time
+            elapsed unless `value` is not datetime.timedelta or cannot be
+            converted to int. In that case, a `value` is returned unchanged.
+
+    Raises:
+        OverflowError: If `value` is too large to convert to datetime.timedelta.
 
     Examples
         Compare two timestamps in a custom local timezone::
@@ -107,16 +120,22 @@ def naturaldelta(value, months=True, minimum_unit="seconds", when=None):
         now = dt.datetime.now(tz=berlin)
         later = now + dt.timedelta(minutes=30)
 
-        assert naturaldelta(later, when=now) == "30 minutes"
+        assert naturaldelta(later - now) == "30 minutes"
     """
     tmp = Unit[minimum_unit.upper()]
     if tmp not in (Unit.SECONDS, Unit.MILLISECONDS, Unit.MICROSECONDS):
-        raise ValueError(f"Minimum unit '{minimum_unit}' not supported")
-    minimum_unit = tmp
+        msg = f"Minimum unit '{minimum_unit}' not supported"
+        raise ValueError(msg)
+    min_unit = tmp
 
-    date, delta = date_and_delta(value, now=when)
-    if date is None:
-        return value
+    if isinstance(value, dt.timedelta):
+        delta = value
+    else:
+        try:
+            value = int(value)
+            delta = dt.timedelta(seconds=value)
+        except (ValueError, TypeError):
+            return str(value)
 
     use_months = months
 
@@ -124,79 +143,99 @@ def naturaldelta(value, months=True, minimum_unit="seconds", when=None):
     days = abs(delta.days)
     years = days // 365
     days = days % 365
-    months = int(days // 30.5)
+    num_months = int(days // 30.5)
 
     if not years and days < 1:
         if seconds == 0:
-            if minimum_unit == Unit.MICROSECONDS and delta.microseconds < 1000:
+            if min_unit == Unit.MICROSECONDS and delta.microseconds < 1000:
                 return (
-                    ngettext("%d microsecond", "%d microseconds", delta.microseconds)
+                    _ngettext("%d microsecond", "%d microseconds", delta.microseconds)
                     % delta.microseconds
                 )
-            elif minimum_unit == Unit.MILLISECONDS or (
-                minimum_unit == Unit.MICROSECONDS
-                and 1000 <= delta.microseconds < 1_000_000
+
+            if min_unit == Unit.MILLISECONDS or (
+                min_unit == Unit.MICROSECONDS and 1000 <= delta.microseconds < 1_000_000
             ):
                 milliseconds = delta.microseconds / 1000
                 return (
-                    ngettext("%d millisecond", "%d milliseconds", milliseconds)
+                    _ngettext("%d millisecond", "%d milliseconds", int(milliseconds))
                     % milliseconds
                 )
             return _("a moment")
-        elif seconds == 1:
+
+        if seconds == 1:
             return _("a second")
-        elif seconds < 60:
-            return ngettext("%d second", "%d seconds", seconds) % seconds
-        elif 60 <= seconds < 120:
+
+        if seconds < 60:
+            return _ngettext("%d second", "%d seconds", seconds) % seconds
+
+        if 60 <= seconds < 120:
             return _("a minute")
-        elif 120 <= seconds < 3600:
+
+        if 120 <= seconds < 3600:
             minutes = seconds // 60
-            return ngettext("%d minute", "%d minutes", minutes) % minutes
-        elif 3600 <= seconds < 3600 * 2:
+            return _ngettext("%d minute", "%d minutes", minutes) % minutes
+
+        if 3600 <= seconds < 3600 * 2:
             return _("an hour")
-        elif 3600 < seconds:
+
+        if 3600 < seconds:
             hours = seconds // 3600
-            return ngettext("%d hour", "%d hours", hours) % hours
+            return _ngettext("%d hour", "%d hours", hours) % hours
+
     elif years == 0:
         if days == 1:
             return _("a day")
+
         if not use_months:
-            return ngettext("%d day", "%d days", days) % days
-        else:
-            if not months:
-                return ngettext("%d day", "%d days", days) % days
-            elif months == 1:
-                return _("a month")
-            else:
-                return ngettext("%d month", "%d months", months) % months
+            return _ngettext("%d day", "%d days", days) % days
+
+        if not num_months:
+            return _ngettext("%d day", "%d days", days) % days
+
+        if num_months == 1:
+            return _("a month")
+
+        return _ngettext("%d month", "%d months", num_months) % num_months
+
     elif years == 1:
-        if not months and not days:
+        if not num_months and not days:
             return _("a year")
-        elif not months:
-            return ngettext("1 year, %d day", "1 year, %d days", days) % days
-        elif use_months:
-            if months == 1:
+
+        if not num_months:
+            return _ngettext("1 year, %d day", "1 year, %d days", days) % days
+
+        if use_months:
+            if num_months == 1:
                 return _("1 year, 1 month")
-            else:
-                return (
-                    ngettext("1 year, %d month", "1 year, %d months", months) % months
-                )
-        else:
-            return ngettext("1 year, %d day", "1 year, %d days", days) % days
-    else:
-        return ngettext("%d year", "%d years", years) % years
+
+            return (
+                _ngettext("1 year, %d month", "1 year, %d months", num_months)
+                % num_months
+            )
+
+        return _ngettext("1 year, %d day", "1 year, %d days", days) % days
+
+    return _ngettext("%d year", "%d years", years).replace("%d", "%s") % intcomma(years)
 
 
-def naturaltime(value, future=False, months=True, minimum_unit="seconds", when=None):
+def naturaltime(
+    value: dt.datetime | dt.timedelta | float,
+    future: bool = False,
+    months: bool = True,
+    minimum_unit: str = "seconds",
+    when: dt.datetime | None = None,
+) -> str:
     """Return a natural representation of a time in a resolution that makes sense.
 
     This is more or less compatible with Django's `naturaltime` filter.
 
     Args:
-        value (datetime.datetime, int): A `datetime` or a number of seconds.
-        future (bool): Ignored for `datetime`s, where the tense is always figured out
-            based on the current time. For integers, the return value will be past tense
-            by default, unless future is `True`.
+        value (datetime.datetime, datetime.timedelta, int or float): A `datetime`, a
+            `timedelta`, or a number of seconds.
+        future (bool): Ignored for `datetime`s and `timedelta`s, where the tense is
+            always figured out based on the current time. For integers and floats, the
+            return value will be past tense by default, unless future is `True`.
         months (bool): If `True`, then a number of months (based on 30.5 days) will be
             used for fuzziness between years.
         minimum_unit (str): The lowest unit that can be used.
@@ -207,23 +246,27 @@ def naturaltime(value, future=False, months=True, minimum_unit="seconds", when=N
         str: A natural representation of the input in a resolution that makes sense.
     """
     now = when or _now()
-    date, delta = date_and_delta(value, now=now)
+
+    if isinstance(value, dt.datetime) and value.tzinfo is not None:
+        value = dt.datetime.fromtimestamp(value.timestamp())
+
+    date, delta = _date_and_delta(value, now=now)
     if date is None:
-        return value
+        return str(value)
     # determine tense by value only if datetime/timedelta were passed
     if isinstance(value, (dt.datetime, dt.timedelta)):
         future = date > now
 
     ago = _("%s from now") if future else _("%s ago")
-    delta = naturaldelta(delta, months, minimum_unit, when=when)
+    delta = naturaldelta(delta, months, minimum_unit)
 
     if delta == _("a moment"):
         return _("now")
 
-    return ago % delta
+    return str(ago % delta)
 
 
-def naturalday(value, format="%b %d"):
+def naturalday(value: dt.date | dt.datetime, format: str = "%b %d") -> str:
     """Return a natural day.
 
     For date values that are tomorrow, today or yesterday compared to
@@ -235,37 +278,47 @@ def naturalday(value, format="%b %d"):
         value = dt.date(value.year, value.month, value.day)
     except AttributeError:
         # Passed value wasn't date-ish
-        return value
+        return str(value)
     except (OverflowError, ValueError):
         # Date arguments out of range
-        return value
+        return str(value)
     delta = value - dt.date.today()
+
     if delta.days == 0:
         return _("today")
-    elif delta.days == 1:
+
+    if delta.days == 1:
         return _("tomorrow")
-    elif delta.days == -1:
+
+    if delta.days == -1:
         return _("yesterday")
+
     return value.strftime(format)
 
 
-def naturaldate(value):
+def naturaldate(value: dt.date | dt.datetime) -> str:
     """Like `naturalday`, but append a year for dates more than ~five months away."""
     try:
         value = dt.date(value.year, value.month, value.day)
     except AttributeError:
         # Passed value wasn't date-ish
-        return value
+        return str(value)
     except (OverflowError, ValueError):
         # Date arguments out of range
-        return value
-    delta = abs_timedelta(value - dt.date.today())
+        return str(value)
+    delta = _abs_timedelta(value - dt.date.today())
     if delta.days >= 5 * 365 / 12:
         return naturalday(value, "%b %d %Y")
     return naturalday(value)
 
 
-def _quotient_and_remainder(value, divisor, unit, minimum_unit, suppress):
+def _quotient_and_remainder(
+    value: float,
+    divisor: float,
+    unit: Unit,
+    minimum_unit: Unit,
+    suppress: collections.abc.Iterable[Unit],
+) -> tuple[float, float]:
     """Divide `value` by `divisor` returning the quotient and remainder.
 
     If `unit` is `minimum_unit`, makes the quotient a float number and the remainder
@@ -291,14 +344,22 @@ def _quotient_and_remainder(value, divisor, unit, minimum_unit, suppress):
 
     """
     if unit == minimum_unit:
-        return (value / divisor, 0)
-    elif unit in suppress:
-        return (0, value)
-    else:
-        return divmod(value, divisor)
+        return value / divisor, 0
+
+    if unit in suppress:
+        return 0, value
+
+    return divmod(value, divisor)
 
 
-def _carry(value1, value2, ratio, unit, min_unit, suppress):
+def _carry(
+    value1: float,
+    value2: float,
+    ratio: float,
+    unit: Unit,
+    min_unit: Unit,
+    suppress: typing.Iterable[Unit],
+) -> tuple[float, float]:
     """Return a tuple with two values.
 
     If the unit is in `suppress`, multiply `value1` by `ratio` and add it to `value2`
@@ -322,60 +383,65 @@ def _carry(value1, value2, ratio, unit, min_unit, suppress):
     (2, 6)
     """
     if unit == min_unit:
-        return (value1 + value2 / ratio, 0)
-    elif unit in suppress:
-        return (0, value2 + value1 * ratio)
-    else:
-        return (value1, value2)
+        return value1 + value2 / ratio, 0
+
+    if unit in suppress:
+        return 0, value2 + value1 * ratio
+
+    return value1, value2
 
 
-def _suitable_minimum_unit(min_unit, suppress):
+def _suitable_minimum_unit(min_unit: Unit, suppress: typing.Iterable[Unit]) -> Unit:
     """Return a minimum unit suitable that is not suppressed.
 
     If not suppressed, return the same unit:
 
     >>> from humanize.time import _suitable_minimum_unit, Unit
-    >>> _suitable_minimum_unit(Unit.HOURS, [])
-    <Unit.HOURS: 4>
+    >>> _suitable_minimum_unit(Unit.HOURS, []).name
+    'HOURS'
 
-    But if suppressed, find a unit greather than the original one that is not
+    But if suppressed, find a unit greater than the original one that is not
     suppressed:
 
-    >>> _suitable_minimum_unit(Unit.HOURS, [Unit.HOURS])
-    <Unit.DAYS: 5>
+    >>> _suitable_minimum_unit(Unit.HOURS, [Unit.HOURS]).name
+    'DAYS'
 
-    >>> _suitable_minimum_unit(Unit.HOURS, [Unit.HOURS, Unit.DAYS])
-    <Unit.MONTHS: 6>
+    >>> _suitable_minimum_unit(Unit.HOURS, [Unit.HOURS, Unit.DAYS]).name
+    'MONTHS'
     """
     if min_unit in suppress:
         for unit in Unit:
             if unit > min_unit and unit not in suppress:
                 return unit
 
-        raise ValueError(
-            "Minimum unit is suppressed and no suitable replacement was found"
-        )
+        msg = "Minimum unit is suppressed and no suitable replacement was found"
+        raise ValueError(msg)
 
     return min_unit
 
 
-def _suppress_lower_units(min_unit, suppress):
+def _suppress_lower_units(min_unit: Unit, suppress: typing.Iterable[Unit]) -> set[Unit]:
     """Extend suppressed units (if any) with all units lower than the minimum unit.
 
     >>> from humanize.time import _suppress_lower_units, Unit
-    >>> list(sorted(_suppress_lower_units(Unit.SECONDS, [Unit.DAYS])))
-    [<Unit.MICROSECONDS: 0>, <Unit.MILLISECONDS: 1>, <Unit.DAYS: 5>]
+    >>> [x.name for x in sorted(_suppress_lower_units(Unit.SECONDS, [Unit.DAYS]))]
+    ['MICROSECONDS', 'MILLISECONDS', 'DAYS']
     """
     suppress = set(suppress)
-    for u in Unit:
-        if u == min_unit:
+    for unit in Unit:
+        if unit == min_unit:
             break
-        suppress.add(u)
+        suppress.add(unit)
 
     return suppress
 
 
-def precisedelta(value, minimum_unit="seconds", suppress=(), format="%0.2f"):
+def precisedelta(
+    value: dt.timedelta | int | None,
+    minimum_unit: str = "seconds",
+    suppress: typing.Iterable[str] = (),
+    format: str = "%0.2f",
+) -> str:
     """Return a precise representation of a timedelta.
 
     ```pycon
@@ -442,21 +508,21 @@ def precisedelta(value, minimum_unit="seconds", suppress=(), format="%0.2f"):
 
     ```
     """
-    date, delta = date_and_delta(value)
+    date, delta = _date_and_delta(value)
     if date is None:
-        return value
+        return str(value)
 
-    suppress = [Unit[s.upper()] for s in suppress]
+    suppress_set = {Unit[s.upper()] for s in suppress}
 
     # Find a suitable minimum unit (it can be greater the one that the
     # user gave us if it is suppressed).
     min_unit = Unit[minimum_unit.upper()]
-    min_unit = _suitable_minimum_unit(min_unit, suppress)
+    min_unit = _suitable_minimum_unit(min_unit, suppress_set)
     del minimum_unit
 
     # Expand the suppressed units list/set to include all the units
     # that are below the minimum unit
-    suppress = _suppress_lower_units(min_unit, suppress)
+    suppress_set = _suppress_lower_units(min_unit, suppress_set)
 
     # handy aliases
     days = delta.days
@@ -479,27 +545,27 @@ def precisedelta(value, minimum_unit="seconds", suppress=(), format="%0.2f"):
     #       years, days = divmod(years, days)
     #
     # The same applies for months, hours, minutes and milliseconds below
-    years, days = _quotient_and_remainder(days, 365, YEARS, min_unit, suppress)
-    months, days = _quotient_and_remainder(days, 30.5, MONTHS, min_unit, suppress)
+    years, days = _quotient_and_remainder(days, 365, YEARS, min_unit, suppress_set)
+    months, days = _quotient_and_remainder(days, 30.5, MONTHS, min_unit, suppress_set)
 
     # If DAYS is not in suppress, we can represent the days but
     # if it is a suppressed unit, we need to carry it to a lower unit,
     # seconds in this case.
     #
     # The same applies for secs and usecs below
-    days, secs = _carry(days, secs, 24 * 3600, DAYS, min_unit, suppress)
+    days, secs = _carry(days, secs, 24 * 3600, DAYS, min_unit, suppress_set)
 
-    hours, secs = _quotient_and_remainder(secs, 3600, HOURS, min_unit, suppress)
-    minutes, secs = _quotient_and_remainder(secs, 60, MINUTES, min_unit, suppress)
+    hours, secs = _quotient_and_remainder(secs, 3600, HOURS, min_unit, suppress_set)
+    minutes, secs = _quotient_and_remainder(secs, 60, MINUTES, min_unit, suppress_set)
 
-    secs, usecs = _carry(secs, usecs, 1e6, SECONDS, min_unit, suppress)
+    secs, usecs = _carry(secs, usecs, 1e6, SECONDS, min_unit, suppress_set)
 
     msecs, usecs = _quotient_and_remainder(
-        usecs, 1000, MILLISECONDS, min_unit, suppress
+        usecs, 1000, MILLISECONDS, min_unit, suppress_set
     )
 
     # if _unused != 0 we had lost some precision
-    usecs, _unused = _carry(usecs, 0, 1, MICROSECONDS, min_unit, suppress)
+    usecs, _unused = _carry(usecs, 0, 1, MICROSECONDS, min_unit, suppress_set)
 
     fmts = [
         ("%d year", "%d years", years),
@@ -512,15 +578,19 @@ def precisedelta(value, minimum_unit="seconds", suppress=(), format="%0.2f"):
         ("%d microsecond", "%d microseconds", usecs),
     ]
 
-    texts = []
+    texts: list[str] = []
     for unit, fmt in zip(reversed(Unit), fmts):
-        singular_txt, plural_txt, value = fmt
-        if value > 0 or (not texts and unit == min_unit):
-            fmt_txt = ngettext(singular_txt, plural_txt, value)
-            if unit == min_unit and math.modf(value)[0] > 0:
+        singular_txt, plural_txt, fmt_value = fmt
+        if fmt_value > 0 or (not texts and unit == min_unit):
+            fmt_txt = _ngettext(singular_txt, plural_txt, fmt_value)
+            if unit == min_unit and math.modf(fmt_value)[0] > 0:
                 fmt_txt = fmt_txt.replace("%d", format)
+            elif unit == YEARS:
+                fmt_txt = fmt_txt.replace("%d", "%s")
+                texts.append(fmt_txt % intcomma(fmt_value))
+                continue
 
-            texts.append(fmt_txt % value)
+            texts.append(fmt_txt % fmt_value)
 
         if unit == min_unit:
             break
